unit OptMedium;


interface

const MaxSpectruNumber=10;

type
  TSpectru=(nd,nC,nF,ng,ne,nh,ni,nr,ns,nt);
          {'nt' 'ns' 'nr' 'nC' 'nd' 'ne' 'nF' 'ng' 'nh' 'ni'}
  TRefrectiveIndexArray=array[nd..nt] of extended;
  TMaxSpectruNumberRange=1..MaxSpectruNumber;

  TOpticalMedium=class
  private
   FDistance              :extended;
   FExchangeDistance      :TRefrectiveIndexArray;
   FRefractiveIndex       :TRefrectiveIndexArray;
   FAbbeNumber            :extended;
   FGlassName             :shortstring;

   function GetExchangeDistance(index:TSpectru):extended;
   procedure SetExchangeDistance(index:TSpectru; value:extended);

   function GetRefractiveIndex(index:TSpectru):extended;
   procedure SetRefractiveIndex(index:TSpectru; value:extended);
  public
   property Distance:extended read FDistance write FDistance;

   property ExchangeDistance[index:TSpectru]:extended read GetExchangeDistance
                                                   write SetExchangeDistance;
   property RefractiveIndex[index:TSpectru]:extended read GetRefractiveIndex
                                                   write SetRefractiveIndex;
   property AbbeNumber:extended read FAbbeNumber write FAbbeNumber;

   property GlassName:shortstring  read FGlassName write  FGlassName;

   procedure Initialize;
   procedure OutputToFile(var OutputFile:file);
   procedure InputFromFile(var InputFile:file);
   procedure InputFromTextFile(var InputFile:Text);
   procedure OutputToTextFile(var OutputFile:Text);

   procedure CalculateExchangeDistance(aN:extended; aSpectru:TSpectru);
   procedure CopyFrom(aMedium:TOpticalMedium);

  end;


implementation
   procedure TOpticalMedium.InputFromFile(var InputFile:file);
   var  n:TSpectru;
   begin
     BlockRead(InputFile, FDistance ,  SizeOf(FDistance ) );
     BlockRead(InputFile, FAbbeNumber ,  SizeOf(FAbbeNumber ) );
     BlockRead(InputFile, FGlassName ,  SizeOf(FGlassName ) );

     for n:=nd to nt do
     begin
       BlockRead(InputFile, FExchangeDistance[n] ,  SizeOf(FExchangeDistance[n] ) );
       BlockRead(InputFile, FRefractiveIndex[n],  SizeOf(FRefractiveIndex[n]) );
     end;

   end;

   procedure TOpticalMedium.OutputToFile(var OutputFile:file);
   var n:TSpectru;
   begin
     BlockWrite(OutputFile, FDistance ,  SizeOf(FDistance ) );
     BlockWrite(OutputFile, FAbbeNumber ,  SizeOf(FAbbeNumber ) );
     BlockWrite(OutputFile, FGlassName ,  SizeOf(FGlassName ) );

     for n:=nd to nt do
     begin
       BlockWrite(OutputFile, FExchangeDistance[n] ,  SizeOf(FExchangeDistance[n] ) );
       BlockWrite(OutputFile, FRefractiveIndex[n],  SizeOf(FRefractiveIndex[n]) );
     end;

   end;


   procedure TOpticalMedium.InputFromTextFile(var InputFile:Text);
   var  n:TSpectru;
   begin
     Readln(InputFile, FDistance);
     Readln(InputFile, FAbbeNumber);
     Readln(InputFile, FGlassName);

     for n:=nd to nt do
     begin
       Readln(InputFile, FExchangeDistance[n] );
       Readln(InputFile, FRefractiveIndex[n]);
     end;

   end;

   procedure TOpticalMedium.OutputToTextFile(var OutputFile:Text);
   var n:TSpectru;
   begin
     Writeln(OutputFile, FDistance);
     Writeln(OutputFile, FAbbeNumber);
     Writeln(OutputFile, FGlassName);

     for n:=nd to nt do
     begin
       Writeln(OutputFile, FExchangeDistance[n]);
       Writeln(OutputFile, FRefractiveIndex[n]);
     end;

   end;


   procedure TOpticalMedium.Initialize;
   var n:TSpectru;
   begin
     FDistance:=0.0;
     FAbbeNumber:=0;
     FGlassName:='air';

     for n:=nd to nt do
     begin
       FExchangeDistance[n]:=0.0;
       FRefractiveIndex[n]:=1.0;
     end;
   end;

   procedure TOpticalMedium.CopyFrom(aMedium:TOpticalMedium);
   var n:TSpectru;
   begin
     FDistance:=aMedium.Distance;
     for n:=nd to nt do
     begin
       FExchangeDistance[n]:=aMedium.ExchangeDistance[n];
       FRefractiveIndex[n]:=aMedium.RefractiveIndex[n];
       FAbbeNumber:=aMedium.AbbeNumber;
       FGlassName:=aMedium.GlassName;
     end;
   end;

   function TOpticalMedium.GetRefractiveIndex(index:TSpectru):extended;
   begin
     result:=FRefractiveIndex[index];
   end;

   procedure TOpticalMedium.SetRefractiveIndex(index:TSpectru;value:extended);
   begin
     FRefractiveIndex[index]:=value;
   end;


   function TOpticalMedium.GetExchangeDistance(index:TSpectru):extended;
   begin
     result:=FExchangeDistance[index];
   end;

   procedure TOpticalMedium.SetExchangeDistance(index:TSpectru;value:extended);
   begin
     FExchangeDistance[index]:=value;
   end;


   procedure TOpticalMedium.CalculateExchangeDistance(aN:extended; aSpectru:TSpectru);
   var
     aE:extended;
   begin
     aE:=Distance / aN;
     ExchangeDistance[aSpectru]:=aE;
   end;

end.



 