unit OptSurface;


interface
uses
  OptMedium,MayMath;

const MaxOptSurface=64;
      FLAT=1.0e4000;
      INFINITY=-1.0e100;  //-1.0e4000;
      MaxOutLine=64;
      MaxCurveLine=31;
      MaxCoefficient=32;
type
  TMaxOptSurfaceRange=1..MaxOptSurface;
  TMatrix=array[1..3,1..3]of extended;
  TVector=array['X'..'Z']of extended;
  TCoefficient=array[1..MaxCoefficient]of extended;
  TFigure=(Round,Rect,Oval,UpperOval);   {'Round','Rect','Oval','UpperOval'}
  TFlagValue=0..3;
  TAspFlagValue=0..2 ;
  TMaxMediumNumberRange=0..MaxOptSurface;

  TOpticalSurface=class
  private
    FCurvature            :extended;
    FRadius               :extended;
    FDiameter             :extended;
    FPower                :TRefrectiveIndexArray;
    FPowerZ               :TRefrectiveIndexArray;
    FConicCoefficient     :extended;
    FTiltSheeta           :extended;
    FTiltPhai             :extended;
    FBiasSheeta           :extended;
    FBiasPhai             :extended;
    FTiltMatrix           :TMatrix;
    FSiftVector           :TVector;
    FBiasVector           :TVector;
    FBiasSiftVector       :TVector;
    FAsphericalTerm       :byte;
    FAsphericalCoefficient:TCoefficient;
    FHighOrderTerm        :byte;
    FHighOrderCoefficient :TCoefficient;
    FHighOrderAngle       :extended;
    FHighOrderSin         :extended;
    FHighOrderCos         :extended;
    FHighOrderPowerY      :TCoefficient;
    FHighOrderPowerZ      :TCoefficient;
    FCylinderCz           :extended;
    FCylinderAngle        :extended;
    FCylinderSin          :extended;
    FCylinderCos          :extended;
    FApertureFigure       :TFigure{string;} ;
    FApertureBrockout     :extended;
    FApertureY            :extended;
    FApertureZ            :extended;
    FApertureSiftY        :extended;
    FApertureSiftZ        :extended;
    FApertureSin          :extended;
    FApertureCos          :extended;
    FFrenelZeroZorn       :extended;
    FFrenelPich           :extended;
    FFrenelBack           :extended;
    FTiltSwitch           :boolean;
    FSiftSwitch           :boolean;
    FBiasSwitch           :boolean;
    FFrenelFlag           :TFlagValue;
    FAspFlag              :TAspFlagValue;
    FHiOrderSwitch        :boolean;
    FConicSwitch          :boolean;
    FCylinderFlag         :TFlagValue;
    FApfigureSwitch       :boolean;
    FReflectSwitch        :boolean;
    FApproxChangeFlag     :TFlagValue;

    FMeridionalLine :array[1..MaxCurveLine,'X'..'Z'] of longint;
    FSagittalLine   :array[1..MaxCurveLine,'X'..'Z'] of longint;
    FOutLine        :array[1..MaxOutLine,'X'..'Z'] of longint;

    FMLineReal :array[1..MaxCurveLine,'X'..'Z'] of extended;
    FSLineReal :array[1..MaxCurveLine,'X'..'Z'] of extended;
    FOLineReal :array[1..MaxOutLine,'X'..'Z'] of extended;

    function GetPower(index:TSpectru):extended;
    procedure SetPower(index:TSpectru;value:extended);

    function GetPowerZ(index:TSpectru):extended;
    procedure SetPowerZ(index:TSpectru;value:extended);

    function GetTiltMatrix(index1:byte;index2:byte):extended;
    procedure SetTiltMatrix(index1:byte;index2:byte;value:extended);

    function GetSiftVector(index:char):extended;
    procedure SetSiftVector(index:char;value:extended);

    function GetBiasVector(index:char):extended;
    procedure SetBiasVector(index:char;value:extended);

    function GetBiasSiftVector(index:char):extended;
    procedure SetBiasSiftVector(index:char;value:extended);

    function GetAsphericalCoefficient(index:byte):extended;
    procedure SetAsphericalCoefficient(index:byte;value:extended);
    function GetHighOrderCoefficient(index:byte):extended;
    procedure SetHighOrderCoefficient(index:byte;value:extended);

    function GetHighOrderPowerY(index:byte):extended;
    procedure SetHighOrderPowerY(index:byte;value:extended);
    function GetHighOrderPowerZ (index:byte):extended;
    procedure SetHighOrderPowerZ(index:byte;value:extended);

    function GetMeridionalLine(index:byte;index2:char) :longInt;
    procedure SetMeridionalLine(index:byte;index2:char;value:longInt);

    function GetSagittalLine(index:byte;index2:char) :longInt;
    procedure SetSagittalLine(index:byte;index2:char;value:longInt);

    function GetOutLine(index:byte;index2:char) :longInt;
    procedure SetOutLine(index:byte;index2:char;value:longInt);

    function GetMLineReal(index:byte;index2:char) :extended;
    procedure SetMLineReal(index:byte;index2:char;value:extended);

    function GetSLineReal(index:byte;index2:char) :extended;
    procedure SetSLineReal(index:byte;index2:char;value:extended);

    function GetOLineReal(index:byte;index2:char) :extended;
    procedure SetOLineReal(index:byte;index2:char;value:extended);



  public
    property Curvature            :extended read FCurvature write FCurvature ;
    property Radius               :extended read FRadius write FRadius ;
    property Diameter             :extended read FDiameter write FDiameter ;
    property Power[index:Tspectru]:extended read GetPower write SetPower ;
    property PowerZ[index:Tspectru]:extended read GetPowerZ write SetPowerZ ;
    property ConicCoefficient     :extended read FConicCoefficient write FConicCoefficient ;
    property TiltSheeta           :extended read FTiltSheeta write FTiltSheeta;
    property TiltPhai             :extended read FTiltPhai  write FTiltPhai  ;
    property BiasSheeta           :extended read FBiasSheeta write FBiasSheeta;
    property BiasPhai             :extended read FBiasPhai  write FBiasPhai ;

    property TiltMatrix[index1:byte;index2:byte]:extended read GetTiltMatrix write SetTiltMatrix;
    property SiftVector[index:char]:extended read GetSiftVector write SetSiftVector;
    property BiasVector[index:char]:extended read GetBiasVector write SetBiasVector;
    property BiasSiftVector[index:char]:extended read GetBiasSiftVector write SetBiasSiftVector;

    property AsphericalTerm       :byte read FAsphericalTerm  write FAsphericalTerm  ;

    property AsphericalCoefficient[index:byte]:extended read GetAsphericalCoefficient write SetAsphericalCoefficient;

    property HighOrderTerm        :byte read FHighOrderTerm write FHighOrderTerm ;

    property HighOrderCoefficient[index:byte]:extended read GetHighOrderCoefficient write SetHighOrderCoefficient;


    property HighOrderAngle       :extended read FHighOrderAngle write FHighOrderAngle ;
    property HighOrderSin         :extended read FHighOrderSin write FHighOrderSin ;
    property HighOrderCos         :extended read FHighOrderCos write FHighOrderCos ;

    property HighOrderPowerY[index:byte]:extended read GetHighOrderPowerY write SetHighOrderPowerY ;
    property HighOrderPowerZ[index:byte]:extended read GetHighOrderPowerZ write SetHighOrderPowerZ ;

    property CylinderCz           :extended read FCylinderCz write FCylinderCz;
    property CylinderAngle        :extended read FCylinderAngle write FCylinderAngle ;
    property CylinderSin          :extended read FCylinderSin write FCylinderSin ;
    property CylinderCos          :extended read FCylinderCos write FCylinderCos ;
    property ApertureFigure       :TFigure read FApertureFigure write FApertureFigure ;
    property ApertureBrockout     :extended read FApertureBrockout write FApertureBrockout ;
    property ApertureY            :extended read FApertureY write FApertureY ;
    property ApertureZ            :extended read FApertureZ write FApertureZ ;
    property ApertureSiftY        :extended read FApertureSiftY write FApertureSiftY ;
    property ApertureSiftZ        :extended read FApertureSiftZ write FApertureSiftZ ;
    property ApertureSin          :extended read FApertureSin write FApertureSin ;
    property ApertureCos          :extended read FApertureCos write FApertureCos ;
    property FrenelZeroZorn       :extended read FFrenelZeroZorn write FFrenelZeroZorn ;
    property FrenelPich           :extended read FFrenelPich write FFrenelPich;
    property FrenelBack           :extended read FFrenelBack write FFrenelBack;
    property TiltSwitch           :boolean read FTiltSwitch write FTiltSwitch ;
    property SiftSwitch           :boolean read FSiftSwitch write FSiftSwitch ;
    property BiasSwitch           :boolean read FBiasSwitch  write FBiasSwitch  ;
    property FrenelFlag           :TFlagValue read FFrenelFlag write FFrenelFlag ;
    property AspFlag              :TAspFlagValue read FAspFlag write FAspFlag ;
    property HiOrderSwitch        :boolean read FHiOrderSwitch write FHiOrderSwitch ;
    property ConicSwitch          :boolean read FConicSwitch write FConicSwitch ;
    property CylinderFlag         :TFlagValue read FCylinderFlag write FCylinderFlag ;
    property ApfigureSwitch       :boolean read FApfigureSwitch write FApfigureSwitch ;
    property ReflectSwitch        :boolean read FReflectSwitch write FReflectSwitch ;
    property ApproxChangeFlag     :TFlagValue read FApproxChangeFlag write FApproxChangeFlag ;

    property MeridionalLine[index:byte;index2:char]:longInt read GetMeridionalLine write SetMeridionalLine;
    property SagittalLine[index:byte;index2:char]:longInt read GetSagittalLine write SetSagittalLine;
    property OutLine[index:byte;index2:char]:longInt read GetOutLine write SetOutLine;
    property MLineReal[index:byte;index2:char]:extended read GetMLineReal write SetMLineReal;
    property SLineReal[index:byte;index2:char]:extended read GetSLineReal write SetSLineReal;
    property OLineReal[index:byte;index2:char]:extended read GetOLineReal write SetOLineReal;



    procedure Initialize;
    procedure OutputToTextFile(var OutputFile:Text);
    procedure InputFromTextFile(var InputFile:Text);
    procedure OutputToFile(var OutputFile:file);
    procedure InputFromFile(var InputFile:file);
    procedure CopyFrom(anOpticalSurface:TOpticalSurface);
    procedure CulculateSurfacePoint(magnification :extended);

    function CalculatePower(aN:extended;aNN:extended;aSpectru:TSpectru):boolean;
    function PopTiltMatrix:TMatrix;
    function PopBiasVector:TVector;
    function Epsxyz(vyi:extended; vzi:extended):extended;
    function AsphericalXsum(vyi:extended; vzi:extended):extended;
    function CylinderEpsxyz(vyi:extended; vzi:extended):extended;

    procedure CulculateSurfacePointInReal;
  end;
    procedure TiltMakeRotation( var aMatrix:TMatrix; aVector:TVector);
    procedure BiasMakeRotation( var aMatrix:TMatrix; aVector:TVector);
    procedure CulculateTconvBy(var aTconv:TMatrix; aSinst:extended; aCosst:extended; aT:Tmatrix);
    procedure Rotation(var aX:longint; var aY:longint; var aZ:longint; aMatrix:TMatrix; aFlag:integer);
    procedure RotationReal(var aX:extended; var aY:extended; var aZ:extended; aMatrix:TMatrix; aFlag:integer);

 implementation
    procedure TOpticalSurface.Initialize;
      var i,k:integer;
          n:char;
          m:TSpectru;
      begin
      FCurvature            :=0.0;
      FRadius               :=FLAT;
      FDiameter             :=1.0;

      for m:=nd to nt do
      begin
        FPower[m]:=0.0;
        FPowerZ[m]:=0.0;
      end;

      FConicCoefficient     :=1.0;
      FTiltSheeta           :=0.0;
      FTiltPhai             :=0.0;
      FBiasSheeta           :=0.0;
      FBiasPhai             :=0.0;

      for i:=1 to 3 do
      begin
        for k:=1 to 3 do
        begin
        FTiltMatrix[i,k]:=0.0;
        end;
      end;

      for n:='X' to 'Z' do
      begin
      FSiftVector[n]:=0.0;
      FBiasVector[n]:=0.0;
      FBiasSiftVector[n]:=0.0;
      end;

      FAsphericalTerm:=0;
      FHighOrderTerm:=0;
      FHighOrderAngle       :=0.0;
      FHighOrderSin         :=0.0;
      FHighOrderCos         :=1;

      for i:=1 to MaxCoefficient do
      begin
      FAsphericalCoefficient[i]:=0.0;
      FHighOrderCoefficient[i]:=0.0;
      FHighOrderPowerY[i]:=0.0;
      FHighOrderPowerZ[i]:=0.0;
      end;

      FCylinderCz           :=0.0;
      FCylinderAngle        :=0.0;
      FCylinderSin          :=0.0;
      FCylinderCos          :=1;
      FApertureFigure       :=Round;
      FApertureBrockout     :=0.0;
      FApertureY            :=0.0;
      FApertureZ            :=0.0;
      FApertureSiftY        :=0.0;
      FApertureSiftZ        :=0.0;
      FApertureSin          :=0.0;
      FApertureCos          :=1;
      FFrenelZeroZorn       :=0.0;
      FFrenelPich           :=0.0;
      FFrenelBack           :=0.0;
      FTiltSwitch           :=False;
      FSiftSwitch           :=False;
      FBiasSwitch           :=False;
      FFrenelFlag           :=0;
      FAspFlag              :=0;
      FHiOrderSwitch        :=False;
      FConicSwitch          :=False;
      FCylinderFlag         :=0;
      FApfigureSwitch       :=False;
      FReflectSwitch        :=False;
      FApproxChangeFlag     :=0;
    end;


    procedure TOpticalSurface.InputFromTextFile(var InputFile:Text);
    var i,k:integer;
          n:char;
          m:TSpectru;
        s:ShortString;
    begin
      Readln(InputFile, FCurvature );
      Readln(InputFile, FRadius );
      Readln(InputFile, FDiameter );

      for m:=nd to nt do
      begin
        Readln(InputFile, FPower[m]);
        Readln(InputFile, FPowerZ[m]);
      end;

      Readln(InputFile, FConicCoefficient);
      Readln(InputFile, FTiltSheeta );
      Readln(InputFile, FTiltPhai);
      Readln(InputFile, FBiasSheeta);
      Readln(InputFile, FBiasPhai);

      for i:=1 to 3 do
      begin
        for k:=1 to 3 do
        begin
        Readln(InputFile,FTiltMatrix[i,k]);
        end;
      end;

      for n:='X' to 'Z' do
      begin
      Readln(InputFile, FSiftVector[n]);
      Readln(InputFile, FBiasVector[n]);
      Readln(InputFile, FBiasSiftVector[n] );
      end;

      Readln(InputFile, FAsphericalTerm );
      Readln(InputFile, FHighOrderTerm);
      Readln(InputFile, FHighOrderAngle);
      Readln(InputFile, FHighOrderSin);
      Readln(InputFile, FHighOrderCos);

      if (FAsphericalTerm<>0) then begin
        for i:=1 to FAsphericalTerm do begin
          Readln(InputFile, FAsphericalCoefficient[i] );
        end;
      end;
      if (FHighOrderTerm<>0) then begin
        for i:=1 to FHighOrderTerm do begin
          Readln(InputFile, FHighOrderCoefficient[i]);
          Readln(InputFile, FHighOrderPowerY[i] );
          Readln(InputFile, FHighOrderPowerZ[i] );
        end;
      end;

      Readln(InputFile, FCylinderCz );
      Readln(InputFile, FCylinderAngle);
      Readln(InputFile, FCylinderSin);
      Readln(InputFile, FCylinderCos);

      Readln(InputFile, s);
      if (s='Round') then
        FApertureFigure:=Round
        else if (s='Rect') then
          FApertureFigure:=Rect
          else if (s='Oval') then
            FApertureFigure:=Oval
            else if (s='UpperOval') then
             FApertureFigure:=UpperOval;

      Readln(InputFile, FApertureBrockout);
      Readln(InputFile, FApertureY);
      Readln(InputFile, FApertureZ);
      Readln(InputFile, FApertureSiftY);
      Readln(InputFile, FApertureSiftZ );
      Readln(InputFile, FApertureSin);
      Readln(InputFile, FApertureCos);
      Readln(InputFile, FFrenelZeroZorn);
      Readln(InputFile, FFrenelPich);
      Readln(InputFile, FFrenelBack);

      Readln(InputFile,s);
      if (s='FALSE') then
        FTiltSwitch:=False
        else
          FTiltSwitch:=True;

      Readln(InputFile,s);
      if (s='FALSE') then
        FSiftSwitch:=False
        else
          FSiftSwitch:=True;

      Readln(InputFile,s);
      if (s='FALSE') then
         FBiasSwitch:=False
        else
           FBiasSwitch:=True;

      Readln(InputFile, FFrenelFlag);
      Readln(InputFile, FAspFlag);

      Readln(InputFile,s);
      if (s='FALSE') then
        FHiOrderSwitch:=False
        else
          FHiOrderSwitch:=True;

      Readln(InputFile,s);
      if (s='FALSE') then
        FConicSwitch:=False
        else
          FConicSwitch:=True;

      Readln(InputFile, FCylinderFlag);

      Readln(InputFile,s);
      if (s='FALSE') then
        FApfigureSwitch:=False
        else
          FApfigureSwitch:=True;

      Readln(InputFile,s);
      if (s='FALSE') then
        FReflectSwitch:=False
        else
          FReflectSwitch:=True;

      Readln(InputFile, FApproxChangeFlag);

    end;

    procedure TOpticalSurface.InputFromFile(var InputFile:file);
    var i,k:integer;
          n:char;
          m:TSpectru;
    begin
      BlockRead(InputFile, FCurvature,  SizeOf(FCurvature) );
      BlockRead(InputFile, FRadius,     SizeOf(FRadius) );
      BlockRead(InputFile, FDiameter,   SizeOf(FDiameter) );

      for m:=nd to nt do
      begin
        BlockRead(InputFile, FPower[m],      SizeOf(FPower[m]) );
        BlockRead(InputFile, FPowerZ[m],     SizeOf(FPowerZ[m]) );
      end;

      BlockRead(InputFile, FConicCoefficient, SizeOf(FConicCoefficient) );
      BlockRead(InputFile, FTiltSheeta,       SizeOf(FTiltSheeta) );
      BlockRead(InputFile, FTiltPhai,         SizeOf(FTiltPhai) );
      BlockRead(InputFile, FBiasSheeta,       SizeOf(FBiasSheeta) );
      BlockRead(InputFile, FBiasPhai,         SizeOf(FBiasPhai) );

      for i:=1 to 3 do
      begin
        for k:=1 to 3 do
        begin
        BlockRead(InputFile,FTiltMatrix[i,k],SizeOf(FTiltMatrix[i,k]) );
        end;
      end;

      for n:='X' to 'Z' do
      begin
      BlockRead(InputFile, FSiftVector[n]         ,SizeOf( FSiftVector[n] ) );
      BlockRead(InputFile, FBiasVector[n]         ,SizeOf( FBiasVector[n] ) );
      BlockRead(InputFile, FBiasSiftVector[n]     ,SizeOf( FBiasSiftVector[n] ) );
      end;

      BlockRead(InputFile, FAsphericalTerm,   SizeOf(FAsphericalTerm) );
      BlockRead(InputFile, FHighOrderTerm,    SizeOf(FHighOrderTerm) );
      BlockRead(InputFile, FHighOrderAngle,   SizeOf(FHighOrderAngle) );
      BlockRead(InputFile, FHighOrderSin,     SizeOf(FHighOrderSin) );
      BlockRead(InputFile, FHighOrderCos,     SizeOf(FHighOrderCos) );

     if (FAsphericalTerm<>0) then begin
        for i:=1 to FAsphericalTerm do begin
          BlockRead(InputFile, FAsphericalCoefficient[i],  SizeOf(FAsphericalCoefficient[i]) );
        end;
      end;
      if (FHighOrderTerm<>0) then begin
        for i:=1 to FHighOrderTerm do begin
          BlockRead(InputFile, FHighOrderCoefficient[i],   SizeOf(FHighOrderCoefficient[i]) );
          BlockRead(InputFile, FHighOrderPowerY[i],        SizeOf(FHighOrderPowerY[i]) );
          BlockRead(InputFile, FHighOrderPowerZ[i],        SizeOf(FHighOrderPowerZ[i]) );
       end;
      end;

      BlockRead(InputFile, FCylinderCz,    SizeOf(FCylinderCz) );
      BlockRead(InputFile, FCylinderAngle, SizeOf(FCylinderAngle) );
      BlockRead(InputFile, FCylinderSin,   SizeOf(FCylinderSin) );
      BlockRead(InputFile, FCylinderCos,   SizeOf(FCylinderCos) );
      BlockRead(InputFile, FApertureFigure,    SizeOf(FApertureFigure) );
      BlockRead(InputFile, FApertureBrockout,  SizeOf(FApertureBrockout) );
      BlockRead(InputFile, FApertureY,         SizeOf(FApertureY) );
      BlockRead(InputFile, FApertureZ,         SizeOf(FApertureZ) );
      BlockRead(InputFile, FApertureSiftY,     SizeOf(FApertureSiftY) );
      BlockRead(InputFile, FApertureSiftZ,     SizeOf(FApertureSiftZ) );
      BlockRead(InputFile, FApertureSin,       SizeOf(FApertureSin) );
      BlockRead(InputFile, FApertureCos,       SizeOf(FApertureCos) );
      BlockRead(InputFile, FFrenelZeroZorn,   SizeOf(FFrenelZeroZorn) );
      BlockRead(InputFile, FFrenelPich,       SizeOf(FFrenelPich) );
      BlockRead(InputFile, FFrenelBack,       SizeOf(FFrenelBack) );
      BlockRead(InputFile, FTiltSwitch,     SizeOf(FTiltSwitch) );
      BlockRead(InputFile, FSiftSwitch,     SizeOf(FSiftSwitch) );
      BlockRead(InputFile, FBiasSwitch,     SizeOf(FBiasSwitch) );
      BlockRead(InputFile, FFrenelFlag,        SizeOf(FFrenelFlag) );
      BlockRead(InputFile, FAspFlag,           SizeOf(FAspFlag) );
      BlockRead(InputFile, FHiOrderSwitch,     SizeOf(FHiOrderSwitch) );
      BlockRead(InputFile, FConicSwitch,       SizeOf(FConicSwitch) );
      BlockRead(InputFile, FCylinderFlag,      SizeOf(FCylinderFlag) );
      BlockRead(InputFile, FApfigureSwitch,    SizeOf(FApfigureSwitch) );
      BlockRead(InputFile, FReflectSwitch,     SizeOf(FReflectSwitch) );
      BlockRead(InputFile, FApproxChangeFlag,  SizeOf(FApproxChangeFlag) );
    end;


    procedure TOpticalSurface.OutputToTextFile(var OutputFile:Text);
    var i,k:integer;
          n:char;
          m:TSpectru;
    begin
      Writeln(OutputFile, FCurvature );
      Writeln(OutputFile, FRadius );
      Writeln(OutputFile, FDiameter );

      for m:=nd to nt do
      begin
        Writeln(OutputFile, FPower[m] );
        Writeln(OutputFile, FPowerZ[m] );
      end;

      Writeln(OutputFile, FConicCoefficient );
      Writeln(OutputFile, FTiltSheeta );
      Writeln(OutputFile, FTiltPhai );
      Writeln(OutputFile, FBiasSheeta );
      Writeln(OutputFile, FBiasPhai );

      for i:=1 to 3 do
      begin
        for k:=1 to 3 do
        begin
        Writeln(OutputFile,FTiltMatrix[i,k] );
        end;
      end;

      for n:='X' to 'Z' do
      begin
      Writeln(OutputFile, FSiftVector[n]);
      Writeln(OutputFile, FBiasVector[n] );
      Writeln(OutputFile, FBiasSiftVector[n] );
      end;

      Writeln(OutputFile, FAsphericalTerm);
      Writeln(OutputFile, FHighOrderTerm);
      Writeln(OutputFile, FHighOrderAngle);
      Writeln(OutputFile, FHighOrderSin);
      Writeln(OutputFile, FHighOrderCos);

      if (FAsphericalTerm<>0) then begin
        for i:=1 to FAsphericalTerm do begin
          Writeln(OutputFile, FAsphericalCoefficient[i]);
        end;
      end;
      if (FHighOrderTerm<>0) then begin
        for i:=1 to FHighOrderterm do begin
          Writeln(OutputFile, FHighOrderCoefficient[i]);
          Writeln(OutputFile, FHighOrderPowerY[i]);
          Writeln(OutputFile, FHighOrderPowerZ[i]);
        end;
      end;

      Writeln(OutputFile, FCylinderCz);
      Writeln(OutputFile, FCylinderAngle);
      Writeln(OutputFile, FCylinderSin);
      Writeln(OutputFile, FCylinderCos);

      if (FApertureFigure=Round) then
        Writeln(OutputFile, 'Round')
        else if (FApertureFigure=Rect) then
          Writeln(OutputFile, 'Rect')
          else if (FApertureFigure=Oval) then
            Writeln(OutputFile, 'Oval')
            else if (FApertureFigure=UpperOval) then
              Writeln(OutputFile, 'UpperOval');

      Writeln(OutputFile, FApertureBrockout);
      Writeln(OutputFile, FApertureY);
      Writeln(OutputFile, FApertureZ);
      Writeln(OutputFile, FApertureSiftY);
      Writeln(OutputFile, FApertureSiftZ);
      Writeln(OutputFile, FApertureSin);
      Writeln(OutputFile, FApertureCos);
      Writeln(OutputFile, FFrenelZeroZorn);
      Writeln(OutputFile, FFrenelPich);
      Writeln(OutputFile, FFrenelBack);
      Writeln(OutputFile, FTiltSwitch);
      Writeln(OutputFile, FSiftSwitch);
      Writeln(OutputFile, FBiasSwitch);
      Writeln(OutputFile, FFrenelFlag);
      Writeln(OutputFile, FAspFlag);
      Writeln(OutputFile, FHiOrderSwitch);
      Writeln(OutputFile, FConicSwitch);
      Writeln(OutputFile, FCylinderFlag);
      Writeln(OutputFile, FApfigureSwitch);
      Writeln(OutputFile, FReflectSwitch);
      Writeln(OutputFile, FApproxChangeFlag);
    end;

    procedure TOpticalSurface.OutputToFile(var OutputFile:file);
    var i,k:integer;
          n:char;
          m:TSpectru;
    begin
      BlockWrite(OutputFile, FCurvature,  SizeOf(FCurvature) );
      BlockWrite(OutputFile, FRadius,     SizeOf(FRadius) );
      BlockWrite(OutputFile, FDiameter,   SizeOf(FDiameter) );

      for m:=nd to nt do
      begin
        BlockWrite(OutputFile, FPower[m],      SizeOf(FPower[m]) );
        BlockWrite(OutputFile, FPowerZ[m],     SizeOf(FPowerZ[m]) );
      end;

      BlockWrite(OutputFile, FConicCoefficient, SizeOf(FConicCoefficient) );
      BlockWrite(OutputFile, FTiltSheeta,       SizeOf(FTiltSheeta) );
      BlockWrite(OutputFile, FTiltPhai,         SizeOf(FTiltPhai) );
      BlockWrite(OutputFile, FBiasSheeta,       SizeOf(FBiasSheeta) );
      BlockWrite(OutputFile, FBiasPhai,         SizeOf(FBiasPhai) );

      for i:=1 to 3 do
      begin
        for k:=1 to 3 do
        begin
        BlockWrite(OutputFile,FTiltMatrix[i,k],SizeOf(FTiltMatrix[i,k]) );
        end;
      end;

      for n:='X' to 'Z' do
      begin
      BlockWrite(OutputFile, FSiftVector[n]         ,SizeOf( FSiftVector[n] ) );
      BlockWrite(OutputFile, FBiasVector[n]         ,SizeOf( FBiasVector[n] ) );
      BlockWrite(OutputFile, FBiasSiftVector[n]     ,SizeOf( FBiasSiftVector[n] ) );
      end;

      BlockWrite(OutputFile, FAsphericalTerm,   SizeOf(FAsphericalTerm) );
      BlockWrite(OutputFile, FHighOrderTerm,    SizeOf(FHighOrderTerm) );
      BlockWrite(OutputFile, FHighOrderAngle,   SizeOf(FHighOrderAngle) );
      BlockWrite(OutputFile, FHighOrderSin,     SizeOf(FHighOrderSin) );
      BlockWrite(OutputFile, FHighOrderCos,     SizeOf(FHighOrderCos) );

      if (FAsphericalTerm<>0) then begin
        for i:=1 to FAsphericalTerm do begin
          BlockWrite(OutputFile, FAsphericalCoefficient[i],  SizeOf(FAsphericalCoefficient[i]) );
        end;
      end;
      if (FHighOrderTerm<>0) then begin
        for i:=1 to FHighOrderTerm do begin
          BlockWrite(OutputFile, FHighOrderCoefficient[i],   SizeOf(FHighOrderCoefficient[i]) );
          BlockWrite(OutputFile, FHighOrderPowerY[i],        SizeOf(FHighOrderPowerY[i]) );
          BlockWrite(OutputFile, FHighOrderPowerZ[i],        SizeOf(FHighOrderPowerZ[i]) );
        end;
      end;

      BlockWrite(OutputFile, FCylinderCz,    SizeOf(FCylinderCz) );
      BlockWrite(OutputFile, FCylinderAngle, SizeOf(FCylinderAngle) );
      BlockWrite(OutputFile, FCylinderSin,   SizeOf(FCylinderSin) );
      BlockWrite(OutputFile, FCylinderCos,   SizeOf(FCylinderCos) );
      BlockWrite(OutputFile, FApertureFigure,    SizeOf(FApertureFigure) );
      BlockWrite(OutputFile, FApertureBrockout,  SizeOf(FApertureBrockout) );
      BlockWrite(OutputFile, FApertureY,         SizeOf(FApertureY) );
      BlockWrite(OutputFile, FApertureZ,         SizeOf(FApertureZ) );
      BlockWrite(OutputFile, FApertureSiftY,     SizeOf(FApertureSiftY) );
      BlockWrite(OutputFile, FApertureSiftZ,     SizeOf(FApertureSiftZ) );
      BlockWrite(OutputFile, FApertureSin,       SizeOf(FApertureSin) );
      BlockWrite(OutputFile, FApertureCos,       SizeOf(FApertureCos) );
      BlockWrite(OutputFile, FFrenelZeroZorn,   SizeOf(FFrenelZeroZorn) );
      BlockWrite(OutputFile, FFrenelPich,       SizeOf(FFrenelPich) );
      BlockWrite(OutputFile, FFrenelBack,       SizeOf(FFrenelBack) );
      BlockWrite(OutputFile, FTiltSwitch,     SizeOf(FTiltSwitch) );
      BlockWrite(OutputFile, FSiftSwitch,     SizeOf(FSiftSwitch) );
      BlockWrite(OutputFile, FBiasSwitch,     SizeOf(FBiasSwitch) );
      BlockWrite(OutputFile, FFrenelFlag,        SizeOf(FFrenelFlag) );
      BlockWrite(OutputFile, FAspFlag,           SizeOf(FAspFlag) );
      BlockWrite(OutputFile, FHiOrderSwitch,     SizeOf(FHiOrderSwitch) );
      BlockWrite(OutputFile, FConicSwitch,       SizeOf(FConicSwitch) );
      BlockWrite(OutputFile, FCylinderFlag,      SizeOf(FCylinderFlag) );
      BlockWrite(OutputFile, FApfigureSwitch,    SizeOf(FApfigureSwitch) );
      BlockWrite(OutputFile, FReflectSwitch,     SizeOf(FReflectSwitch) );
      BlockWrite(OutputFile, FApproxChangeFlag,  SizeOf(FApproxChangeFlag) );
    end;


    procedure TOpticalSurface.CopyFrom(anOpticalSurface:TOpticalSurface);
      var i,k:integer;
          n:char;
          m:TSpectru;
      begin
      FCurvature            :=anOpticalSurface.Curvature;
      FRadius               :=anOpticalSurface.Radius;
      FDiameter             :=anOpticalSurface.Diameter;

      for m:=nd to nt do
      begin
        FPower[m]:=anOpticalSurface.Power[m];
        FPowerZ[m]:=anOpticalSurface.PowerZ[m];
      end;

      FConicCoefficient     :=anOpticalSurface.ConicCoefficient;
      FTiltSheeta           :=anOpticalSurface.TiltSheeta ;
      FTiltPhai             :=anOpticalSurface.TiltPhai;
      FBiasSheeta           :=anOpticalSurface.BiasSheeta;
      FBiasPhai             :=anOpticalSurface.BiasPhai;

      for i:=1 to 3 do
      begin
        for k:=1 to 3 do
        begin
        FTiltMatrix[i,k]:=anOpticalSurface.TiltMatrix[i,k];
        end;
      end;

      for n:='X' to 'Z' do
      begin
      FSiftVector[n]:=anOpticalSurface.SiftVector[n];
      FBiasVector[n]:=anOpticalSurface.BiasVector[n];
      FBiasSiftVector[n]:=anOpticalSurface.BiasSiftVector[n];
      end;

      FAsphericalTerm:=anOpticalSurface.AsphericalTerm;
      FHighOrderTerm:=anOpticalSurface.HighOrderTerm;
      FHighOrderAngle       :=anOpticalSurface.HighOrderAngle;
      FHighOrderSin         :=anOpticalSurface.HighOrderSin  ;
      FHighOrderCos         :=anOpticalSurface.HighOrderCos ;

      for i:=1 to MaxCoefficient do
      begin
      FAsphericalCoefficient[i]:=anOpticalSurface.AsphericalCoefficient[i];
      FHighOrderCoefficient[i]:=anOpticalSurface.HighOrderCoefficient[i];
      FHighOrderPowerY[i]:=anOpticalSurface.HighOrderPowerY[i];
      FHighOrderPowerZ[i]:=anOpticalSurface.HighOrderPowerZ[i];
      end;

      FCylinderCz           :=anOpticalSurface.CylinderCz ;
      FCylinderAngle        :=anOpticalSurface.CylinderAngle;
      FCylinderSin          :=anOpticalSurface.CylinderSin;
      FCylinderCos          :=anOpticalSurface.CylinderCos;
      FApertureFigure       :=anOpticalSurface.ApertureFigure;
      FApertureBrockout     :=anOpticalSurface.ApertureBrockout;
      FApertureY            :=anOpticalSurface.ApertureY;
      FApertureZ            :=anOpticalSurface.ApertureZ;
      FApertureSiftY        :=anOpticalSurface.ApertureSiftY;
      FApertureSiftZ        :=anOpticalSurface.ApertureSiftZ;
      FApertureSin          :=anOpticalSurface.ApertureSin;
      FApertureCos          :=anOpticalSurface.ApertureCos;
      FFrenelZeroZorn       :=anOpticalSurface.FrenelZeroZorn;
      FFrenelPich           :=anOpticalSurface.FrenelPich;
      FFrenelBack           :=anOpticalSurface.FrenelBack;
      FTiltSwitch           :=anOpticalSurface.TiltSwitch;
      FSiftSwitch           :=anOpticalSurface.SiftSwitch;
      FBiasSwitch           :=anOpticalSurface.BiasSwitch;
      FFrenelFlag           :=anOpticalSurface.FrenelFlag;
      FAspFlag              :=anOpticalSurface.AspFlag;
      FHiOrderSwitch        :=anOpticalSurface.HiOrderSwitch;
      FConicSwitch          :=anOpticalSurface.ConicSwitch;
      FCylinderFlag         :=anOpticalSurface.CylinderFlag;
      FApfigureSwitch       :=anOpticalSurface.ApfigureSwitch ;
      FReflectSwitch        :=anOpticalSurface.ReflectSwitch ;
      FApproxChangeFlag     :=anOpticalSurface.ApproxChangeFlag;
    end;

    function TOpticalSurface.GetPower(index:TSpectru):extended;
    begin
      result:=FPower[index];
    end;

    procedure TOpticalSurface.SetPower(index:TSpectru;value:extended);
    begin
      FPower[index]:=value;
    end;

    function TOpticalSurface.GetPowerZ(index:TSpectru):extended;
    begin
      result:=FPowerZ[index];
    end;

    procedure TOpticalSurface.SetPowerZ(index:TSpectru;value:extended);
    begin
      FPowerZ[index]:=value;
    end;

    function TOpticalSurface.GetTiltMatrix(index1:byte;index2:byte):extended;
    begin
      result:=FTiltMatrix[index1,index2];
    end;

    procedure TOpticalSurface.SetTiltMatrix(index1:byte;index2:byte;value:extended);
    begin
      FTiltMatrix[index1,index2]:=value;
    end;

    function TOpticalSurface.GetSiftVector(index:char):extended;
    begin
       result:=FSiftVector[index];
    end;

    procedure TOpticalSurface.SetSiftVector(index:char;value:extended);
    begin
       FSiftVector[index]:=value;
    end;

    function TOpticalSurface.GetBiasVector(index:char):extended;
    begin
       result:=FBiasVector[index];
    end;

    procedure TOpticalSurface.SetBiasVector(index:char;value:extended);
    begin
      FBiasVector[index]:=value;
    end;

    function TOpticalSurface.GetBiasSiftVector(index:char):extended;
    begin
      result:=FBiasSiftVector[index];
    end;

    procedure TOpticalSurface.SetBiasSiftVector(index:char;value:extended);
    begin
       FBiasSiftVector[index]:=value;
    end;

    function TOpticalSurface.GetAsphericalCoefficient(index:byte):extended;
    begin
      result:=FAsphericalCoefficient[index];
    end;

    procedure TOpticalSurface.SetAsphericalCoefficient(index:byte;value:extended);
    begin
      FAsphericalCoefficient[index]:=value;
    end;

    function TOpticalSurface.GetHighOrderCoefficient(index:byte):extended;
    begin
       result:=FHighOrderCoefficient[index];
    end;

    procedure TOpticalSurface.SetHighOrderCoefficient(index:byte;value:extended);
    begin
       FHighOrderCoefficient[index]:=value;
    end;

    function TOpticalSurface.GetHighOrderPowerY(index:byte):extended;
    begin
      result:=FHighOrderPowerY[index];
    end;

    procedure TOpticalSurface.SetHighOrderPowerY(index:byte;value:extended);
    begin
       FHighOrderPowerY[index]:=value;
    end;

    function TOpticalSurface.GetHighOrderPowerZ(index:byte):extended;
    begin
      result:=FHighOrderPowerZ[index];
    end;

    procedure TOpticalSurface.SetHighOrderPowerZ(index:byte;value:extended);
    begin
      FHighOrderPowerZ[index]:=value;
    end;

    function TOpticalSurface.GetMLineReal(index:byte;index2:char) :extended;
    begin
      result:=FMLineReal[index,index2];
    end;

    function TOpticalSurface.GetSLineReal(index:byte;index2:char) :extended;
    begin
      result:=FSLineReal[index,index2];
    end;

    function TOpticalSurface.GetOLineReal(index:byte;index2:char) :extended;
    begin
      result:=FOLineReal[index,index2];
    end;

    procedure TOpticalSurface.SetMLineReal(index:byte;index2:char;value:extended);
    begin
      FMLineReal[index,index2]:=value;
    end;

     procedure TOpticalSurface.SetSLineReal(index:byte;index2:char;value:extended);
    begin
      FSLineReal[index,index2]:=value;
    end;

    procedure TOpticalSurface.SetOLineReal(index:byte;index2:char;value:extended);
    begin
      FOLineReal[index,index2]:=value;
    end;


    function TOpticalSurface.GetMeridionalLine(index:byte;index2:char) :longInt;
    begin
      result:=FMeridionalLine[index,index2];
    end;

    procedure TOpticalSurface.SetMeridionalLine(index:byte;index2:char;value:longInt);
    begin
      FMeridionalLine[index,index2]:=value;
    end;

    function TOpticalSurface.GetSagittalLine(index:byte;index2:char) :longInt;
    begin
      result:=FSagittalLine[index,index2];
    end;

    procedure TOpticalSurface.SetSagittalLine(index:byte;index2:char;value:longInt);
    begin
      FSagittalLine[index,index2]:=value;
    end;

    function TOpticalSurface.GetOutLine(index:byte;index2:char) :longInt;
    begin
      result:=FOutLine[index,index2];
    end;

    procedure TOpticalSurface.SetOutLine(index:byte;index2:char;value:longInt);
    begin
      FOutLine[index,index2]:=Value;
    end;

//******************************************************************************

    function TOpticalSurface.CalculatePower(aN:extended;aNN:extended;aSpectru:TSpectru):boolean;
    var
      aPower,aC,aPowerZ,aCz:extended;
    begin

      if ( (aspFlag=0) and (hiOrderSwitch=False) and (cylinderFlag=0) ) then begin
	aC:=Curvature;
	aCz:=Curvature;
	aPower:=(aNN - aN) * aC;
	Power[aSpectru]:=aPower;
	aPowerZ:=(aNN - aN) * aCz;
	PowerZ[aSpectru]:=aPowerZ;
	result:=True;
        exit;
      end else begin
	if AspFlag<>0 then begin
	  aC:=Curvature + ((AsphericalCoefficient[1])*2);
	  aCz:=Curvature + ((AsphericalCoefficient[1])*2);
	  aPower:=(aNN - aN) * aC;
	  Power[aSpectru]:=aPower;
	  aPowerZ:=(aNN - aN) * aCz;
	  PowerZ[aSpectru]:=aPowerZ;
	  result:=True;
          exit;         {  hiOrderSwitch  Ɂ@sȂėǂ̂@}
	end;

	if HiOrderSwitch=True then begin
	  aC:=Curvature + ((HighOrderCoefficient[1])*2);
	  aCz:=Curvature + ((HighOrderCoefficient[1])*2);
	  aPower:=(aNN - aN) * aC;
	  Power[aSpectru]:=aPower;
	  aPowerZ:=(aNN - aN) * aCz;
	  PowerZ[aSpectru]:=aPowerZ;
	  result:=True;
          exit;
	end;

	if CylinderFlag<>0 then begin
	  aC:=Curvature;
	  aCz:=CylinderCz;
	  aPower:=(aNN - aN) * aC;
	  Power[aSpectru]:=aPower;
	  aPowerZ:=(aNN - aN) * aCz;
	  PowerZ[aSpectru]:=aPowerZ;
	  result:=True;
          exit;
	end;

      end;
      result:=False;
    end;

function TOpticalSurface.PopTiltMatrix:TMatrix;
begin
  result:=FTiltMatrix;
end;

function TOpticalSurface.PopBiasVector:TVector;
begin
  result:=FBiasVector;
end;

procedure BiasMakeRotation( var aMatrix:TMatrix; aVector:TVector);
var
 aDdi, aVdsq, aSqxyi, aSqeiy,
 aVex, aVey, aVez, aEx, aEy, aEz:extended;
 i,j:integer;
begin
  aVex:=aVector['X'];
  aVey:=aVector['Y'];
  aVez:=aVector['Z'];
  aDdi:= 1.0 / sqrt(aVex*aVex + aVey*aVey + aVez*aVez);
  aEx:=aVex*aDdi;
  aEy:=aVey*aDdi;
  aEz:=aVez*aDdi;

  aMatrix[1,1]:=aEx;
  aMatrix[1,2]:=aEy;
  aMatrix[1,3]:=aEz;

 if (abs(aEz)>0.8) then begin
    aVdsq:= 1.0 - (aEy*aEy);
    aSqeiy:= 1.0 /sqrt(aVdsq);

    aMatrix[3,1]:=(-1*aEz*aSqeiy);
    aMatrix[3,2]:= 0.0;
    aMatrix[3,3]:=(aEx *aSqeiy);

    aMatrix[2,1]:=-1 * aMatrix[1,2] * aMatrix[3,3];
    aMatrix[2,2]:=aMatrix[1,1]*aMatrix[3,3] - aMatrix[1,3]*aMatrix[3,1];
    aMatrix[2,3]:=aMatrix[1,2] * aMatrix[3,1];
  end else begin
    aVdsq:= 1.0 - (aEz*aEz);
    aSqxyi:= 1.0 /sqrt(aVdsq);

    aMatrix[2,1]:=(-1*aEy*aSqxyi);
    aMatrix[2,2]:=(aEx *aSqxyi);
    aMatrix[2,3]:= 0.0;

    aMatrix[3,1]:= -1 * aMatrix[1,3] * aMatrix[2,2];
    aMatrix[3,2]:=aMatrix[1,3]*aMatrix[2,1];
    aMatrix[3,3]:=aMatrix[1,1] * aMatrix[2,2] - aMatrix[1,2]*aMatrix[2,1];
  end;

  for i:=1 to 3 do begin
    for j:=1 to 3 do begin
      if abs(aMatrix[i,j])<0.0000001 then
        aMatrix[i,j]:=0;
    end;
  end;

end;

procedure TiltMakeRotation( var aMatrix:TMatrix; aVector:TVector);
var
 aDdi, aVdsq, aSqxyi, aSqeiy,
 aVex, aVey, aVez, aEx, aEy, aEz:extended;
 i,j:integer;
begin

  aVex:=aVector['X'];
  aVey:=aVector['Y'];
  aVez:=aVector['Z'];
  aDdi:= 1.0 / sqrt(aVex*aVex + aVey*aVey + aVez*aVez);
  aEx:=aVex*aDdi;
  aEy:=aVey*aDdi;
  aEz:=aVez*aDdi;

  aMatrix[1,1]:=aEx;
  aMatrix[1,2]:=aEy;
  aMatrix[1,3]:=aEz;

 if (abs(aEz)>0.8 {0.99999}  ) then begin
    aVdsq:= 1.0 - (aEy*aEy);
    aSqeiy:= 1.0 /sqrt(aVdsq);

    aMatrix[3,1]:=(-1*aEz*aSqeiy);
    aMatrix[3,2]:= 0.0;
    aMatrix[3,3]:=(aEx *aSqeiy);

    aMatrix[2,1]:=-1 * aMatrix[1,2] * aMatrix[3,3];
    aMatrix[2,2]:=aMatrix[1,1]*aMatrix[3,3] - aMatrix[1,3]*aMatrix[3,1];
    aMatrix[2,3]:=aMatrix[1,2] * aMatrix[3,1];


 {   aMatrix[2,1]:=(-1);
    aMatrix[2,2]:=0.0;
    aMatrix[2,3]:= 0.0;

    aMatrix[3,1]:= -1 * aMatrix[1,3] * aMatrix[2,2];
    aMatrix[3,2]:=aMatrix[1,3]*aMatrix[2,1];
    aMatrix[3,3]:=aMatrix[1,1] * aMatrix[2,2] - aMatrix[1,2]*aMatrix[2,1];
 }

  end else begin
    aVdsq:= 1.0 - (aEz*aEz);
    aSqxyi:= 1.0 /sqrt(aVdsq);

    aMatrix[2,1]:=(-1*aEy*aSqxyi);
    aMatrix[2,2]:=(aEx *aSqxyi);
    aMatrix[2,3]:= 0.0;

    aMatrix[3,1]:= -1 * aMatrix[1,3] * aMatrix[2,2];
    aMatrix[3,2]:=aMatrix[1,3]*aMatrix[2,1];
    aMatrix[3,3]:=aMatrix[1,1] * aMatrix[2,2] - aMatrix[1,2]*aMatrix[2,1];
  end;
  for i:=1 to 3 do begin
    for j:=1 to 3 do begin
      if abs(aMatrix[i,j])<0.0000001 then
        aMatrix[i,j]:=0;
    end;
  end;

end;

procedure CulculateTconvBy(var aTconv:TMatrix; aSinst:extended; aCosst:extended; aT:Tmatrix);{ 3.34 CTCONV }
var
  aTemp:TMatrix;
  aSum:extended;
  jj,mm,n:integer;
begin

  aTemp[1,1]:=1;
  aTemp[1,2]:=0;
  aTemp[1,3]:=0;
  aTemp[2,1]:=0;
  aTemp[2,2]:=aCosst;
  aTemp[2,3]:=( -1 * aSinst );
  aTemp[3,1]:=0;
  aTemp[3,2]:=aSinst;
  aTemp[3,3]:=aCosst;

  for jj:=1 to 3 do begin
    for mm:=1 to 3 do begin
      aSum:=0;
      for n:=1 to 3 do begin
        aSum:=aSum + aT[jj,n] * aTemp[n,mm];
      end;
      aTconv[jj,mm]:=aSum;
    end;
  end;

end;

function TOpticalSurface.Epsxyz(vyi:extended; vzi:extended):extended;{ 3.16 ESPXYZ }
var
  aHsqr, aCsqr, anA12, anEr1, anEpsXyz:extended;
begin

  if (CylinderFlag=0) then begin
    aHsqr:=(vyi * vyi) + (vzi * vzi);
    aCsqr:=sqr(Curvature);
    if ConicSwitch=true then begin
      anA12:=1.0 - (ConicCoefficient * aCsqr * aHsqr);
    end else begin
      anA12:=1.0 - (1.0 * aCsqr * aHsqr);
    end;

    if anA12 < 0 then begin
      anEr1:= 1.0e10;
      if (Curvature > 1.0e-8) then begin
	anEr1:= 1.0 / curvature;
      end;
     anEpsXyz:=anEr1;
     result:=anEpsXyz;
     exit;
    end;

    anEpsXyz:= aHsqr * Curvature / (1.0 + sqrt(anA12));
    if aspFlag<>0 then begin
      anEpsXyz:= anEpsXyz + (self.AsphericalXsum(vyi, vzi));
    end;

  end else begin
    anEpsXyz:=self.CylinderEpsxyz(vyi, vzi);
  end;

  result:=anEpsXyz;

end;


function TOpticalSurface.AsphericalXsum(vyi:extended; vzi:extended):extended;{ 3.16 EPSXYZ 28-54 }
var
  aH2pow, aXsum, aVh1:extended;
  i:integer;
begin

  aH2pow:=(vyi * vyi) + (vzi * vzi);
  if (aspFlag=2) then begin
    aH2pow:=sqrt(( vyi * vyi) + (vzi * vzi ));
  end;

  aXsum:=0;
  aVh1:=1.0;

  for i:=1 to AsphericalTerm do begin
    aVh1:=aVh1*aH2pow;
    aXsum:=aXsum + aVh1*AsphericalCoefficient[i];
  end;

  result:=aXsum;

end;

function TOpticalSurface.CylinderEpsxyz(vyi:extended; vzi:extended):extended;{ 3.16 EPSXYZ 58-87 }
var
  aValcy, aValcz, aValcyz, aVyibar, aVzibar,
  aD12, anEr1, anEpsxyz, aQ12, aDsq12, anAlpha,
  aBunsi1, aBunsi2, aBunsi3, aBunsi:extended;
begin

  aValcy:=Curvature;
  aValcz:=CylinderCz;
  aValcyz:=aValcz * aValcy;

  aVyibar:=(vyi * CylinderCos) + (vzi * CylinderSin);
  aVzibar:=(-1 * vyi * CylinderSin) + (vzi * CylinderCos);
  aD12:=1.0 - ((aVyibar*aValcy) * (aVyibar*aValcy));
  if aD12 < 0 then begin
    anEr1:=1.0e10;
    if (abs(Curvature) > 1.0e-8) then begin
      anEr1:=1.0 / Curvature;
    end;
    anEpsxyz:=anEr1;
    result:=anEpsxyz;
    exit;
  end;

  aDsq12:=sqrt(aD12);
  anAlpha:=1.0 / ( 1.0 + aDsq12 );
  aQ12:=sqr(1.0 - sqr(aVyibar)*aValcyz*anAlpha) - sqr(aVzibar*aValcz);
  if aQ12 < 0 then begin
    anEr1:=1.0e10;
    if abs(Curvature) > 1.0e-8 then begin
      anEr1:=1.0 / Curvature;
    end;
    anEpsxyz:= anEr1;
    result:=anEpsxyz;
    exit;
  end;

  aBunsi1:= 2.0 * (aVyibar*aVyibar) * aValcy * anAlpha;
  aBunsi2:= -1 * (aVyibar*aVyibar*aVyibar*aVyibar) * aValcyz * aValcy * (anAlpha*anAlpha);
  aBunsi3:= (aVzibar*aVzibar) * aValcz;
  aBunsi:= aBunsi1 + aBunsi2 + aBunsi3;

  anEpsxyz:= aBunsi / ( 1.0 + sqrt(aQ12) );
  result:=anEpsxyz;

end;

 procedure TOpticalSurface.CulculateSurfacePoint(magnification :extended);
 var
   i: integer;
   aStep, aDr, A2, B2, aHz, aHy,aStepZ,aStepY,outLineY,outLineZ,
   slY,slZ,mlY,mlZ: extended;
   aY, aZ, aX: extended;
   aMatrix:TMatrix;
 begin
  {'Round','Rect','Oval','UpperOval'}
   if self.ApFigureSwitch=False then begin
     aDr:=self.Diameter/2;
     for i:=1 to MaxOutLine do begin
       aY:=aDr*Cos(2*PI/MaxOutLine*(i-1));
       aZ:=aDr*Sin(2*PI/MaxOutLine*(i-1));
       aX:=self.Epsxyz(aY,aZ);
       OutLine[i,'Y']:=trunc( aY*Magnification );
       OutLine[i,'Z']:=trunc( aZ*Magnification );
       OutLine[i,'X']:=trunc( aX*Magnification );
     end;

     aStep:=self.Diameter/(MaxCurveLine-1);
     aDr:=self.Diameter/2;

     aZ:=0;
     for i:=1 to MaxCurveLine do begin
       aY:=aDr - aStep*(i-1);
       aX:=self.Epsxyz(aY,aZ);
       MeridionalLine[i,'Y']:=trunc( aY*Magnification );
       MeridionalLine[i,'Z']:=0;
       MeridionalLine[i,'X']:=trunc( aX*Magnification  );
     end;

     aY:=0;
     for i:=1 to MaxCurveLine do begin
       aZ:=aDr - aStep*(i-1);
       aX:=self.Epsxyz(aY,aZ);
       SagittalLine[i,'Y']:=0;
       SagittalLine[i,'Z']:=trunc( aZ*Magnification );
       SagittalLine[i,'X']:=trunc( aX*Magnification  );
     end;

   end else begin

   if self.ApertureFigure=Rect then begin
     aStepZ:=self.ApertureZ/((MaxOutLine/4));
     aStepY:=self.ApertureY/((MaxOutLine/4));
     aHz:=self.ApertureZ/2;
     aHy:=self.ApertureY/2;
     for i:=1 to (MaxOutLine Div 4) do begin
       aZ:=aHz;
       aY:=aHy-aStepY*(i-1);
       aX:=self.Epsxyz(aY,aZ);
       OutLine[i,'Y']:=trunc( aY*Magnification );
       OutLine[i,'Z']:=trunc( aZ*Magnification );
       OutLine[i,'X']:=trunc( aX*Magnification );
     end;
     for i:=((MaxOutLine Div 4)+1) to (MaxOutLine Div 4)*2 do begin
       aZ:=aHz-aStepZ*( i-((MaxOutLine Div 4)+1) );
       aY:=-aHy;
       aX:=self.Epsxyz(aY,aZ);
       OutLine[i,'Y']:=trunc( aY*Magnification );
       OutLine[i,'Z']:=trunc( aZ*Magnification );
       OutLine[i,'X']:=trunc( aX*Magnification );
     end;
     for i:=((MaxOutLine div 4)*2+1) to (MaxOutLine div 4)*3 do begin
       aZ:=-aHz;
       aY:=-aHy+aStepY*( i-((MaxOutLine div 4)*2+1) );
       aX:=self.Epsxyz(aY,aZ);
       OutLine[i,'Y']:=trunc( aY*Magnification );
       OutLine[i,'Z']:=trunc( aZ*Magnification );
       OutLine[i,'X']:=trunc( aX*Magnification );
     end;
     for i:=((MaxOutLine div 4)*3+1) to (MaxOutLine div 4)*4 do begin
       aZ:=-aHz+aStepZ*( i-((MaxOutLine div 4)*3+1) );
       aY:=aHy;
       aX:=self.Epsxyz(aY,aZ);
       OutLine[i,'Y']:=trunc( aY*Magnification );
       OutLine[i,'Z']:=trunc( aZ*Magnification );
       OutLine[i,'X']:=trunc( aX*Magnification );
     end;

     for i:=1 to MaxOutLine do begin
       OutLine[i,'Y']:=OutLine[i,'Y']+trunc(
                              self.ApertureSiftY*Magnification);
       OutLine[i,'Z']:=OutLine[i,'Z']+trunc(
                              self.ApertureSiftZ*Magnification);
     end;
     for i:=1 to MaxOutLine do begin
      outLineY:=OutLine[i,'Y'];
      outLineZ:=OutLine[i,'Z'];

       OutLine[i,'Y']:=trunc(
                              ((outLineY*self.ApertureCos)
                              + (-1*outLineZ *self.ApertureSin))
                               );
       OutLine[i,'Z']:=trunc(
                              ((outLineY*self.ApertureSin)
                              + (outLineZ *self.ApertureCos))
                              );


     end;

     aStep:=self.ApertureY/(MaxCurveLine-1);
     aDr:=self.ApertureY/2;
     aZ:=0;
     for i:=1 to MaxCurveLine do begin
       aY:=aDr - aStep*(i-1);
       aX:=self.Epsxyz(aY,aZ);
       MeridionalLine[i,'Y']:=trunc( aY*Magnification );
       MeridionalLine[i,'Z']:=0;
       MeridionalLine[i,'X']:=trunc( aX*Magnification  );
     end;

     aStep:=self.ApertureZ/(MaxCurveLine-1);
     aDr:=self.ApertureZ/2;
     aY:=0;
     for i:=1 to MaxCurveLine do begin
       aZ:=aDr - aStep*(i-1);
       aX:=self.Epsxyz(aY,aZ);
       SagittalLine[i,'Y']:=0;
       SagittalLine[i,'Z']:=trunc( aZ*Magnification );
       SagittalLine[i,'X']:=trunc( aX*Magnification  );
     end;
     for i:=1 to MaxCurveLine do begin
       MeridionalLine[i,'Y']:=MeridionalLine[i,'Y']+trunc(
                             self.ApertureSiftY*Magnification);
       MeridionalLine[i,'Z']:=MeridionalLine[i,'Z']+trunc(
                              self.ApertureSiftZ*Magnification);
       SagittalLine[i,'Y']:=SagittalLine[i,'Y']+trunc(
                              self.ApertureSiftY*Magnification);
       SagittalLine[i,'Z']:=SagittalLine[i,'Z']+trunc(
                              self.ApertureSiftZ*Magnification);
     end;
     for i:=1 to MaxCurveLine do begin
       mlY:=  MeridionalLine[i,'Y'];
       mlZ:=  MeridionalLine[i,'Z'];
       slY:=  SagittalLine[i,'Y'] ;
       slZ:=  SagittalLine[i,'Z'];

       MeridionalLine[i,'Y']:=trunc(
                              ((mlY*self.ApertureCos)
                              + (-1*mlZ *self.ApertureSin))
                               );
       MeridionalLine[i,'Z']:=trunc(
                              (( mlY *self.ApertureSin)
                              + (mlZ * self.ApertureCos))
                               );
       SagittalLine[i,'Y']:=trunc(
                              ((slY*self.ApertureCos)
                              + (-1*slZ * self.ApertureSin))
                              );
       SagittalLine[i,'Z']:=trunc(
                              ((slY *self.ApertureSin)
                              + (slZ *self.ApertureCos))
                               );
     end;


   end;

   if self.ApertureFigure=Oval then begin
     aStep:=self.ApertureZ/(MaxOutLine/2);
     aHz:=self.ApertureZ/2;
     A2:=sqr(self.ApertureZ/2);
     B2:=sqr(self.ApertureY/2);

     for i:=1 to (MaxOutLine DIV 2) do begin
       aZ:=aHz-aStep*(i-1);
       aY:=sqrt( B2 - B2/A2*sqr(aZ) );
       aX:=self.Epsxyz(aY,aZ);
       OutLine[i,'Y']:=trunc( aY*Magnification );
       OutLine[i,'Z']:=trunc( aZ*Magnification );
       OutLine[i,'X']:=trunc( aX*Magnification );
     end;
     for i:=(MaxOutLine DIV 2)+1 to MaxOutLine do begin
       aZ:=-aHz+aStep*(i-(MaxOutLine/2+1));
       aY:=-sqrt( B2 - B2/A2*sqr(aZ) );
       aX:=self.Epsxyz(aY,aZ);
       OutLine[i,'Y']:=trunc( aY*Magnification );
       OutLine[i,'Z']:=trunc( aZ*Magnification );
       OutLine[i,'X']:=trunc( aX*Magnification );
     end;

     for i:=1 to MaxOutLine do begin
       OutLine[i,'Y']:=OutLine[i,'Y']+trunc(
                              self.ApertureSiftY*Magnification);
       OutLine[i,'Z']:=OutLine[i,'Z']+trunc(
                              self.ApertureSiftZ*Magnification);
     end;
     for i:=1 to MaxOutLine do begin
      outLineY:=OutLine[i,'Y'];
      outLineZ:=OutLine[i,'Z'];

       OutLine[i,'Y']:=trunc(
                              ((outLineY*self.ApertureCos)
                              + (-1*outLineZ * self.ApertureSin))
                               );
       OutLine[i,'Z']:=trunc(
                              ((outLineY*self.ApertureSin)
                              + (outLineZ * self.ApertureCos))
                              );
     end;

     aStep:=self.ApertureY/(MaxCurveLine-1);
     aDr:=self.ApertureY/2;
     aZ:=0;
     for i:=1 to MaxCurveLine do begin
       aY:=aDr - aStep*(i-1);
       aX:=self.Epsxyz(aY,aZ);
       MeridionalLine[i,'Y']:=trunc( aY*Magnification );
       MeridionalLine[i,'Z']:=0;
       MeridionalLine[i,'X']:=trunc( aX*Magnification  );
     end;

     aStep:=self.ApertureZ/(MaxCurveLine-1);
     aDr:=self.ApertureZ/2;
     aY:=0;
     for i:=1 to MaxCurveLine do begin
       aZ:=aDr - aStep*(i-1);
       aX:=self.Epsxyz(aY,aZ);
       SagittalLine[i,'Y']:=0;
       SagittalLine[i,'Z']:=trunc( aZ*Magnification );
       SagittalLine[i,'X']:=trunc( aX*Magnification  );
     end;
     for i:=1 to MaxCurveLine do begin
       MeridionalLine[i,'Y']:=MeridionalLine[i,'Y']+trunc(
                             self.ApertureSiftY*Magnification);
       MeridionalLine[i,'Z']:=MeridionalLine[i,'Z']+trunc(
                              self.ApertureSiftZ*Magnification);
       SagittalLine[i,'Y']:=SagittalLine[i,'Y']+trunc(
                              self.ApertureSiftY*Magnification);
       SagittalLine[i,'Z']:=SagittalLine[i,'Z']+trunc(
                              self.ApertureSiftZ*Magnification);
     end;
     for i:=1 to MaxCurveLine do begin
       mlY:=  MeridionalLine[i,'Y'];
       mlZ:=  MeridionalLine[i,'Z'];
       slY:=  SagittalLine[i,'Y'] ;
       slZ:=  SagittalLine[i,'Z'];

       MeridionalLine[i,'Y']:=trunc(
                              ((mlY*self.ApertureCos)
                              + (-1*mlZ *self.ApertureSin))
                               );
       MeridionalLine[i,'Z']:=trunc(
                              ((mlY *self.ApertureSin)
                              + (mlZ * self.ApertureCos))
                               );
       SagittalLine[i,'Y']:=trunc(
                              ((slY*self.ApertureCos)
                              + (-1*slZ * self.ApertureSin))
                              );
       SagittalLine[i,'Z']:=trunc(
                              ((slY *self.ApertureSin)
                              + (slZ *self.ApertureCos))
                               );
     end;


   end;

   if self.ApertureFigure=UpperOval then begin
     aStep:=self.ApertureZ/(MaxOutLine/2);
     aHz:=self.ApertureZ/2;
     A2:=sqr(self.ApertureZ/2);
     B2:=sqr(self.ApertureY/2);

     for i:=1 to (MaxOutLine DIV 2) do begin
       aZ:=aHz-aStep*(i-1);
       aY:=sqrt( B2 - B2/A2*sqr(aZ) );
       aX:=self.Epsxyz(aY,aZ);
       OutLine[i,'Y']:=trunc( aY*Magnification );
       OutLine[i,'Z']:=trunc( aZ*Magnification );
       OutLine[i,'X']:=trunc( aX*Magnification );
     end;
     for i:=(MaxOutLine DIV 2)+1 to MaxOutLine do begin
       aZ:=-aHz+aStep*(i-(MaxOutLine/2+1));
       aY:=0;
       aX:=self.Epsxyz(aY,aZ);
       OutLine[i,'Y']:=trunc( aY*Magnification );
       OutLine[i,'Z']:=trunc( aZ*Magnification );
       OutLine[i,'X']:=trunc( aX*Magnification );
     end;
     for i:=1 to MaxOutLine do begin
       OutLine[i,'Y']:=OutLine[i,'Y']+trunc(
                              self.ApertureSiftY*Magnification);
       OutLine[i,'Z']:=OutLine[i,'Z']+trunc(
                              self.ApertureSiftZ*Magnification);
     end;
     for i:=1 to MaxOutLine do begin
       outLineY:=OutLine[i,'Y'];
       outLineZ:=OutLine[i,'Z'];

       OutLine[i,'Y']:=trunc(
                              ((outLineY*self.ApertureCos)
                              + (-1*outLineZ * self.ApertureSin))
                               );
       OutLine[i,'Z']:=trunc(
                              ((outLineY*self.ApertureSin)
                              + (outLineZ * self.ApertureCos))
                              );

     end;

     aStep:=self.ApertureY/(MaxCurveLine-1);
     aDr:=self.ApertureY/2;
     aZ:=0;
     for i:=1 to (MaxCurveLine div 2)+1 do begin
       aY:=aDr - aStep*(i-1);
       aX:=self.Epsxyz(aY,aZ);
       MeridionalLine[i,'Y']:=trunc( aY*Magnification );
       MeridionalLine[i,'Z']:=0;
       MeridionalLine[i,'X']:=trunc( aX*Magnification  );
     end;
     for i:=(MaxCurveLine div 2)+2 to MaxCurveLine do begin
       aY:=0;
       aX:=self.Epsxyz(aY,aZ);
       MeridionalLine[i,'Y']:=trunc( aY*Magnification );
       MeridionalLine[i,'Z']:=0;
       MeridionalLine[i,'X']:=trunc( aX*Magnification  );
     end;

     aStep:=self.ApertureZ/(MaxCurveLine-1);
     aDr:=self.ApertureZ/2;
     aY:=0;
     for i:=1 to MaxCurveLine do begin
       aZ:=aDr - aStep*(i-1);
       aX:=self.Epsxyz(aY,aZ);
       SagittalLine[i,'Y']:=0;
       SagittalLine[i,'Z']:=trunc( aZ*Magnification );
       SagittalLine[i,'X']:=trunc( aX*Magnification  );
     end;
     for i:=1 to MaxCurveLine do begin
       MeridionalLine[i,'Y']:=MeridionalLine[i,'Y']+trunc(
                             self.ApertureSiftY*Magnification);
       MeridionalLine[i,'Z']:=MeridionalLine[i,'Z']+trunc(
                              self.ApertureSiftZ*Magnification);
       SagittalLine[i,'Y']:=SagittalLine[i,'Y']+trunc(
                              self.ApertureSiftY*Magnification);
       SagittalLine[i,'Z']:=SagittalLine[i,'Z']+trunc(
                              self.ApertureSiftZ*Magnification);
     end;
     for i:=1 to MaxCurveLine do begin
       mlY:=  MeridionalLine[i,'Y'];
       mlZ:=  MeridionalLine[i,'Z'];
       slY:=  SagittalLine[i,'Y'] ;
       slZ:=  SagittalLine[i,'Z'];

       MeridionalLine[i,'Y']:=trunc(
                              ((mlY*self.ApertureCos)
                              + (-1*mlZ *self.ApertureSin))
                               );
       MeridionalLine[i,'Z']:=trunc(
                              ((mlY *self.ApertureSin)
                              + (mlZ * self.ApertureCos))
                               );
       SagittalLine[i,'Y']:=trunc(
                              ((slY*self.ApertureCos)
                              + (-1*slZ * self.ApertureSin))
                              );
       SagittalLine[i,'Z']:=trunc(
                              ((slY *self.ApertureSin)
                              + (slZ *self.ApertureCos))
                               );
     end;


   end;
   end;
   
   if self.BiasSwitch=True then begin

     BiasMakeRotation(aMatrix, self.PopBiasVector);
     for i:=1 to MaxOutLine do begin
       Rotation( FOutLine[i,'X'], FOutLine[i,'Y'], FOutLine[i,'Z'],aMatrix,1);
     end;
     for i:=1 to MaxCurveLine do begin
       Rotation(FMeridionalLine[i,'X'],FMeridionalLine[i,'Y'],FMeridionalLine[i,'Z'],aMatrix,1);
       Rotation(FSagittalLine[i,'X'],FSagittalLine[i,'Y'],FSagittalLine[i,'Z'],aMatrix,1);
     end;

     for i:=1 to MaxOutLine do begin
       OutLine[i,'X']:=OutLine[i,'X']
                       +trunc(self.BiasSiftVector['X']*Magnification);
       OutLine[i,'Y']:=OutLine[i,'Y']
                       +trunc(self.BiasSiftVector['Y']*Magnification);
       OutLine[i,'Z']:=OutLine[i,'Z']
                       +trunc(self.BiasSiftVector['Z']*Magnification);
     end;
     for i:=1 to MaxCurveLine do begin
       MeridionalLine[i,'X']:=MeridionalLine[i,'X']
                              +trunc(self.BiasSiftVector['X']*Magnification);
       MeridionalLine[i,'Y']:=MeridionalLine[i,'Y']
                              +trunc(self.BiasSiftVector['Y']*Magnification);
       MeridionalLine[i,'Z']:=MeridionalLine[i,'Z']
                              +trunc(self.BiasSiftVector['Z']*Magnification);

       SagittalLine[i,'X']:=SagittalLine[i,'X']
                              +trunc(self.BiasSiftVector['X']*Magnification);
       SagittalLine[i,'Y']:=SagittalLine[i,'Y']
                              +trunc(self.BiasSiftVector['Y']*Magnification);
       SagittalLine[i,'Z']:=SagittalLine[i,'Z']
                              +trunc(self.BiasSiftVector['Z']*Magnification);
     end;

   end;


 end;


 procedure TOpticalSurface.CulculateSurfacePointInReal;
 var
   i: integer;
   aStep, aDr, A2, B2, aHz, aHy,aStepZ,aStepY,outLineY,outLineZ,
   slY,slZ,mlY,mlZ, aSwell: extended;
   aY, aZ, aX: extended;
   aMatrix:TMatrix;
 begin
  {'Round','Rect','Oval','UpperOval'}
   if self.ApFigureSwitch=False then begin
     aDr:=self.Diameter/2;
     for i:=1 to MaxOutLine do begin
       aY:=aDr*Cos(2*PI/MaxOutLine*(i-1));
       aZ:=aDr*Sin(2*PI/MaxOutLine*(i-1));
       aX:=self.Epsxyz(aY,aZ);
       OLineReal[i,'Y']:= aY;
       OLineReal[i,'Z']:= aZ;
       OLineReal[i,'X']:= aX;
     end;

     aStep:=self.Diameter/(MaxCurveLine-1);
     aDr:=self.Diameter/2;

     aZ:=0;
     for i:=1 to MaxCurveLine do begin
       aY:=aDr - aStep*(i-1);
       if self.FrenelFlag = 0 then begin
         aX:=self.Epsxyz(aY,aZ);
       end;
       if(self.FrenelFlag = 1) or (self.FrenelFlag = 2) then begin
         if  abs(aY)<abs(self.frenelZeroZorn) then begin
           aX:=self.Epsxyz(aY,aZ);
         end else begin
           aSwell:=self.Epsxyz(self.frenelZeroZorn,0);
           if aSwell>0 then
             aX:=0
           else
             aX:=aSwell;
         end;
       end;

       MLineReal[i,'Y']:= aY;
       MLineReal[i,'Z']:=0;
       MLineReal[i,'X']:= aX;
     end;

     aY:=0;
     for i:=1 to MaxCurveLine do begin
       aZ:=aDr - aStep*(i-1);
       if self.FrenelFlag = 0 then begin
         aX:=self.Epsxyz(aY,aZ);
       end;
       if(self.FrenelFlag = 1) or (self.FrenelFlag = 2) then begin
         if  abs(aZ)<abs(self.frenelZeroZorn) then begin
           aX:=self.Epsxyz(aY,aZ);
         end else begin
           aSwell:=self.Epsxyz(self.frenelZeroZorn,0);
           if aSwell>0 then
             aX:=0
           else
             aX:=aSwell;
         end;
       end;

       SLineReal[i,'Y']:=0;
       SLineReal[i,'Z']:= aZ;
       SLineReal[i,'X']:= aX;
     end;

   end else begin

   if self.ApertureFigure=Rect then begin
     aStepZ:=self.ApertureZ/((MaxOutLine/4));
     aStepY:=self.ApertureY/((MaxOutLine/4));
     aHz:=self.ApertureZ/2;
     aHy:=self.ApertureY/2;
     for i:=1 to (MaxOutLine Div 4) do begin
       aZ:=aHz;
       aY:=aHy-aStepY*(i-1);
       aX:=self.Epsxyz(aY,aZ);
       OLineReal[i,'Y']:= aY;
       OLineReal[i,'Z']:= aZ;
       OLineReal[i,'X']:= aX;
     end;
     for i:=((MaxOutLine Div 4)+1) to (MaxOutLine Div 4)*2 do begin
       aZ:=aHz-aStepZ*( i-((MaxOutLine Div 4)+1) );
       aY:=-aHy;
       aX:=self.Epsxyz(aY,aZ);
       OLineReal[i,'Y']:= aY;
       OLineReal[i,'Z']:= aZ;
       OLineReal[i,'X']:= aX;
     end;
     for i:=((MaxOutLine div 4)*2+1) to (MaxOutLine div 4)*3 do begin
       aZ:=-aHz;
       aY:=-aHy+aStepY*( i-((MaxOutLine div 4)*2+1) );
       aX:=self.Epsxyz(aY,aZ);
       OLineReal[i,'Y']:= aY;
       OLineReal[i,'Z']:= aZ;
       OLineReal[i,'X']:= aX;
     end;
     for i:=((MaxOutLine div 4)*3+1) to (MaxOutLine div 4)*4 do begin
       aZ:=-aHz+aStepZ*( i-((MaxOutLine div 4)*3+1) );
       aY:=aHy;
       aX:=self.Epsxyz(aY,aZ);
       OLineReal[i,'Y']:= aY;
       OLineReal[i,'Z']:= aZ;
       OLineReal[i,'X']:= aX;
     end;

     for i:=1 to MaxOutLine do begin
       OLineReal[i,'Y']:=OLineReal[i,'Y']+
                              self.ApertureSiftY;
       OLineReal[i,'Z']:=OLineReal[i,'Z']+
                              self.ApertureSiftZ;
     end;
     for i:=1 to MaxOutLine do begin
      outLineY:=OLineReal[i,'Y'];
      outLineZ:=OLineReal[i,'Z'];

       OLineReal[i,'Y']:=((outLineY*self.ApertureCos)
                              + (-1*outLineZ *self.ApertureSin));
       OLineReal[i,'Z']:=((outLineY*self.ApertureSin)
                              + (outLineZ *self.ApertureCos));


     end;

     aStep:=self.ApertureY/(MaxCurveLine-1);
     aDr:=self.ApertureY/2;
     aZ:=0;
     for i:=1 to MaxCurveLine do begin
       aY:=aDr - aStep*(i-1);
       if self.FrenelFlag = 0 then begin
         aX:=self.Epsxyz(aY,aZ);
       end;
       if(self.FrenelFlag = 1) or (self.FrenelFlag = 2) then begin
         if  abs(aY)<abs(self.frenelZeroZorn) then begin
           aX:=self.Epsxyz(aY,aZ);
         end else begin
           aSwell:=self.Epsxyz(self.frenelZeroZorn,0);
           if aSwell>0 then
             aX:=0
           else
             aX:=aSwell;
         end;
       end;

       MLineReal[i,'Y']:= aY;
       MLineReal[i,'Z']:=0;
       MLineReal[i,'X']:= aX;
     end;

     aStep:=self.ApertureZ/(MaxCurveLine-1);
     aDr:=self.ApertureZ/2;
     aY:=0;
     for i:=1 to MaxCurveLine do begin
       aZ:=aDr - aStep*(i-1);
       if self.FrenelFlag = 0 then begin
         aX:=self.Epsxyz(aY,aZ);
       end;
       if(self.FrenelFlag = 1) or (self.FrenelFlag = 2) then begin
         if  abs(aZ)<abs(self.frenelZeroZorn) then begin
           aX:=self.Epsxyz(aY,aZ);
         end else begin
           aSwell:=self.Epsxyz(self.frenelZeroZorn,0);
           if aSwell>0 then
             aX:=0
           else
             aX:=aSwell;
         end;
       end;

       SLineReal[i,'Y']:=0;
       SLineReal[i,'Z']:= aZ;
       SLineReal[i,'X']:= aX;
     end;

     for i:=1 to MaxCurveLine do begin
       MLineReal[i,'Y']:=MLineReal[i,'Y']+self.ApertureSiftY;
       MLineReal[i,'Z']:=MLineReal[i,'Z']+self.ApertureSiftZ;
       SLineReal[i,'Y']:=SLineReal[i,'Y']+self.ApertureSiftY;
       SLineReal[i,'Z']:=SLineReal[i,'Z']+self.ApertureSiftZ;
     end;
     for i:=1 to MaxCurveLine do begin
       mlY:=  MLineReal[i,'Y'];
       mlZ:=  MLineReal[i,'Z'];
       slY:=  SLineReal[i,'Y'] ;
       slZ:=  SLineReal[i,'Z'];

       MLineReal[i,'Y']:=((mlY*self.ApertureCos)+ (-1*mlZ *self.ApertureSin));
       MLineReal[i,'Z']:=(( mlY *self.ApertureSin)+ (mlZ * self.ApertureCos));
       SLineReal[i,'Y']:=((slY*self.ApertureCos)+ (-1*slZ * self.ApertureSin));
       SLineReal[i,'Z']:=((slY *self.ApertureSin)+ (slZ *self.ApertureCos));
     end;


   end;

   if self.ApertureFigure=Oval then begin
     aStep:=self.ApertureZ/(MaxOutLine/2);
     aHz:=self.ApertureZ/2;
     A2:=sqr(self.ApertureZ/2);
     B2:=sqr(self.ApertureY/2);

     for i:=1 to (MaxOutLine DIV 2) do begin
       aZ:=aHz-aStep*(i-1);
       aY:=sqrt( B2 - B2/A2*sqr(aZ) );
       aX:=self.Epsxyz(aY,aZ);
       OLineReal[i,'Y']:= aY;
       OLineReal[i,'Z']:= aZ;
       OLineReal[i,'X']:= aX;
     end;
     for i:=(MaxOutLine DIV 2)+1 to MaxOutLine do begin
       aZ:=-aHz+aStep*(i-(MaxOutLine/2+1));
       aY:=-sqrt( B2 - B2/A2*sqr(aZ) );
       aX:=self.Epsxyz(aY,aZ);
       OLineReal[i,'Y']:= aY;
       OLineReal[i,'Z']:= aZ;
       OLineReal[i,'X']:= aX;
     end;

     for i:=1 to MaxOutLine do begin
       OLineReal[i,'Y']:=OLineReal[i,'Y']+self.ApertureSiftY;
       OLineReal[i,'Z']:=OLineReal[i,'Z']+self.ApertureSiftZ;
     end;
     for i:=1 to MaxOutLine do begin
      outLineY:=OLineReal[i,'Y'];
      outLineZ:=OLineReal[i,'Z'];

       OLineReal[i,'Y']:=((outLineY*self.ApertureCos)+ (-1*outLineZ * self.ApertureSin));
       OLineReal[i,'Z']:=((outLineY*self.ApertureSin)+ (outLineZ * self.ApertureCos));
     end;

     aStep:=self.ApertureY/(MaxCurveLine-1);
     aDr:=self.ApertureY/2;
     aZ:=0;
     for i:=1 to MaxCurveLine do begin
       aY:=aDr - aStep*(i-1);
       if self.FrenelFlag = 0 then begin
         aX:=self.Epsxyz(aY,aZ);
       end;
       if(self.FrenelFlag = 1) or (self.FrenelFlag = 2) then begin
         if  abs(aY)<abs(self.frenelZeroZorn) then begin
           aX:=self.Epsxyz(aY,aZ);
         end else begin
           aSwell:=self.Epsxyz(self.frenelZeroZorn,0);
           if aSwell>0 then
             aX:=0
           else
             aX:=aSwell;
         end;
       end;

       MLineReal[i,'Y']:= aY;
       MLineReal[i,'Z']:=0;
       MLineReal[i,'X']:= aX;
     end;

     aStep:=self.ApertureZ/(MaxCurveLine-1);
     aDr:=self.ApertureZ/2;
     aY:=0;
     for i:=1 to MaxCurveLine do begin
       aZ:=aDr - aStep*(i-1);
       if self.FrenelFlag = 0 then begin
         aX:=self.Epsxyz(aY,aZ);
       end;
       if(self.FrenelFlag = 1) or (self.FrenelFlag = 2) then begin
         if  abs(aZ)<abs(self.frenelZeroZorn) then begin
           aX:=self.Epsxyz(aY,aZ);
         end else begin
           aSwell:=self.Epsxyz(self.frenelZeroZorn,0);
           if aSwell>0 then
             aX:=0
           else
             aX:=aSwell;
         end;
       end;

       SLineReal[i,'Y']:=0;
       SLineReal[i,'Z']:= aZ;
       SLineReal[i,'X']:= aX;
     end;

     for i:=1 to MaxCurveLine do begin
       MLineReal[i,'Y']:=MLineReal[i,'Y']+self.ApertureSiftY;
       MLineReal[i,'Z']:=MLineReal[i,'Z']+self.ApertureSiftZ;
       SLineReal[i,'Y']:=SLineReal[i,'Y']+self.ApertureSiftY;
       SLineReal[i,'Z']:=SLineReal[i,'Z']+self.ApertureSiftZ;
     end;
     for i:=1 to MaxCurveLine do begin
       mlY:=  MLineReal[i,'Y'];
       mlZ:=  MLineReal[i,'Z'];
       slY:=  SLineReal[i,'Y'] ;
       slZ:=  SLineReal[i,'Z'];

       MLineReal[i,'Y']:=((mlY*self.ApertureCos)+ (-1*mlZ *self.ApertureSin));
       MLineReal[i,'Z']:=((mlY *self.ApertureSin)+ (mlZ * self.ApertureCos));
       SLineReal[i,'Y']:=((slY*self.ApertureCos)+ (-1*slZ * self.ApertureSin));
       SLineReal[i,'Z']:=((slY *self.ApertureSin)+ (slZ *self.ApertureCos));
     end;


   end;

   if self.ApertureFigure=UpperOval then begin
     aStep:=self.ApertureZ/(MaxOutLine/2);
     aHz:=self.ApertureZ/2;
     A2:=sqr(self.ApertureZ/2);
     B2:=sqr(self.ApertureY/2);

     for i:=1 to (MaxOutLine DIV 2) do begin
       aZ:=aHz-aStep*(i-1);
       aY:=sqrt( B2 - B2/A2*sqr(aZ) );
       aX:=self.Epsxyz(aY,aZ);
       OLineReal[i,'Y']:= aY;
       OLineReal[i,'Z']:= aZ;
       OLineReal[i,'X']:= aX;
     end;
     for i:=(MaxOutLine DIV 2)+1 to MaxOutLine do begin
       aZ:=-aHz+aStep*(i-(MaxOutLine/2+1));
       aY:=0;
       aX:=self.Epsxyz(aY,aZ);
       OLineReal[i,'Y']:= aY;
       OLineReal[i,'Z']:= aZ;
       OLineReal[i,'X']:= aX;
     end;
     for i:=1 to MaxOutLine do begin
       OLineReal[i,'Y']:=OLineReal[i,'Y']+self.ApertureSiftY;
       OLineReal[i,'Z']:=OLineReal[i,'Z']+self.ApertureSiftZ;
     end;
     for i:=1 to MaxOutLine do begin
       outLineY:=OLineReal[i,'Y'];
       outLineZ:=OLineReal[i,'Z'];

       OLineReal[i,'Y']:=((outLineY*self.ApertureCos)+ (-1*outLineZ * self.ApertureSin));
       OLineReal[i,'Z']:=((outLineY*self.ApertureSin)+ (outLineZ * self.ApertureCos));

     end;

     aStep:=self.ApertureY/(MaxCurveLine-1);
     aDr:=self.ApertureY/2;
     aZ:=0;
     for i:=1 to (MaxCurveLine div 2)+1 do begin
       aY:=aDr - aStep*(i-1);
       aX:=self.Epsxyz(aY,aZ);
       MLineReal[i,'Y']:= aY;
       MLineReal[i,'Z']:=0;
       MLineReal[i,'X']:= aX;
     end;
     for i:=(MaxCurveLine div 2)+2 to MaxCurveLine do begin
       aY:=0;
       aX:=self.Epsxyz(aY,aZ);
       MLineReal[i,'Y']:= aY;
       MLineReal[i,'Z']:=0;
       MLineReal[i,'X']:= aX;
     end;

     aStep:=self.ApertureZ/(MaxCurveLine-1);
     aDr:=self.ApertureZ/2;
     aY:=0;
     for i:=1 to MaxCurveLine do begin
       aZ:=aDr - aStep*(i-1);
       aX:=self.Epsxyz(aY,aZ);
       SLineReal[i,'Y']:=0;
       SLineReal[i,'Z']:= aZ;
       SLineReal[i,'X']:= aX;
     end;
     for i:=1 to MaxCurveLine do begin
       MLineReal[i,'Y']:=MLineReal[i,'Y']+self.ApertureSiftY;
       MLineReal[i,'Z']:=MLineReal[i,'Z']+self.ApertureSiftZ;
       SLineReal[i,'Y']:=SLineReal[i,'Y']+self.ApertureSiftY;
       SLineReal[i,'Z']:=SLineReal[i,'Z']+self.ApertureSiftZ;
     end;
     for i:=1 to MaxCurveLine do begin
       mlY:=  MLineReal[i,'Y'];
       mlZ:=  MLineReal[i,'Z'];
       slY:=  SLineReal[i,'Y'] ;
       slZ:=  SLineReal[i,'Z'];

       MLineReal[i,'Y']:=((mlY*self.ApertureCos)+ (-1*mlZ *self.ApertureSin));
       MLineReal[i,'Z']:=((mlY *self.ApertureSin)+ (mlZ * self.ApertureCos));
       SLineReal[i,'Y']:=((slY*self.ApertureCos)+ (-1*slZ * self.ApertureSin));
       SLineReal[i,'Z']:=((slY *self.ApertureSin)+ (slZ *self.ApertureCos));
     end;


   end;
   end;
   
   if self.BiasSwitch=True then begin

     BiasMakeRotation(aMatrix, self.PopBiasVector);
     for i:=1 to MaxOutLine do begin
       RotationReal( FOLineReal[i,'X'], FOLineReal[i,'Y'], FOLineReal[i,'Z'],aMatrix,1);
     end;
     for i:=1 to MaxCurveLine do begin
       RotationReal(FMLineReal[i,'X'],FMLineReal[i,'Y'],FMLineReal[i,'Z'],aMatrix,1);
       RotationReal(FSLineReal[i,'X'],FSLineReal[i,'Y'],FSLineReal[i,'Z'],aMatrix,1);
     end;

     for i:=1 to MaxOutLine do begin
       OLineReal[i,'X']:=OLineReal[i,'X']+self.BiasSiftVector['X'];
       OLineReal[i,'Y']:=OLineReal[i,'Y']+self.BiasSiftVector['Y'];
       OLineReal[i,'Z']:=OLineReal[i,'Z']+self.BiasSiftVector['Z'];
     end;
     for i:=1 to MaxCurveLine do begin
       MLineReal[i,'X']:=MLineReal[i,'X']+self.BiasSiftVector['X'];
       MLineReal[i,'Y']:=MLineReal[i,'Y']+self.BiasSiftVector['Y'];
       MLineReal[i,'Z']:=MLineReal[i,'Z']+self.BiasSiftVector['Z'];

       SLineReal[i,'X']:=SLineReal[i,'X']+self.BiasSiftVector['X'];
       SLineReal[i,'Y']:=SLineReal[i,'Y']+self.BiasSiftVector['Y'];
       SLineReal[i,'Z']:=SLineReal[i,'Z']+self.BiasSiftVector['Z'];
     end;

   end;


 end;



procedure {TOpticalSurface.}Rotation(var aX:longint; var aY:longint; var aZ:longint; aMatrix:TMatrix; aFlag:integer);
var
  aStockDataX, aStockDataY, aStockDataZ:longint;
begin

  aStockDataX:=aX;
  aStockDataY:=aY;
  aStockDataZ:=aZ;
  if aFlag=0 then begin
   aX:=trunc(aMatrix[1,1]*aStockDataX + aMatrix[1,2]*aStockDataY + aMatrix[1,3]*aStockDataZ);
   aY:=trunc(aMatrix[2,1]*aStockDataX + aMatrix[2,2]*aStockDataY + aMatrix[2,3]*aStockDataZ);
   aZ:=trunc(aMatrix[3,1]*aStockDataX + aMatrix[3,2]*aStockDataY + aMatrix[3,3]*aStockDataZ);
  end else begin
   aX:=trunc(aMatrix[1,1]*aStockDataX + aMatrix[2,1]*aStockDataY + aMatrix[3,1]*aStockDataZ);
   aY:=trunc(aMatrix[1,2]*aStockDataX + aMatrix[2,2]*aStockDataY + aMatrix[3,2]*aStockDataZ);
   aZ:=trunc(aMatrix[1,3]*aStockDataX + aMatrix[2,3]*aStockDataY + aMatrix[3,3]*aStockDataZ);
  end;

end;


procedure {TOpticalSurface.}RotationReal(var aX:extended; var aY:extended; var aZ:extended; aMatrix:TMatrix; aFlag:integer);
var
  aStockDataX, aStockDataY, aStockDataZ:extended;
begin

  aStockDataX:=aX;
  aStockDataY:=aY;
  aStockDataZ:=aZ;
  if aFlag=0 then begin
   aX:=(aMatrix[1,1]*aStockDataX + aMatrix[1,2]*aStockDataY + aMatrix[1,3]*aStockDataZ);
   aY:=(aMatrix[2,1]*aStockDataX + aMatrix[2,2]*aStockDataY + aMatrix[2,3]*aStockDataZ);
   aZ:=(aMatrix[3,1]*aStockDataX + aMatrix[3,2]*aStockDataY + aMatrix[3,3]*aStockDataZ);
  end else begin
   aX:=(aMatrix[1,1]*aStockDataX + aMatrix[2,1]*aStockDataY + aMatrix[3,1]*aStockDataZ);
   aY:=(aMatrix[1,2]*aStockDataX + aMatrix[2,2]*aStockDataY + aMatrix[3,2]*aStockDataZ);
   aZ:=(aMatrix[1,3]*aStockDataX + aMatrix[2,3]*aStockDataY + aMatrix[3,3]*aStockDataZ);
  end;

end;

end.


